function output=evaluator
% output=evaluator
%
% output.solution (solution candidate: threshold value or scaling factor)
% output.accepted (1 (model is valid) or 0 (not))
% output.value (the basis for acceptance: Kolmogorov-Smirnov P value)
% output.success (the value considered to be a measure of the solution's success: interval width)
% output.more_info (additional information about the solution: model parameters, truncation thresholds, etc.)
%
% 'evaluator' evaluates the median of the set of candidate solutions, SCS, as a solution, updates the SCS based on the result, and returns the result.
% The global variables that it needs are defined in the function 'ke_compute'.
%
% Author: Murat Okatan, PhD.
% Citation: Okatan M, Kocatrk M. Truncation thresholds: a pair of spike detection thresholds computed using truncated probability distributions. 
% Turk J Elec Eng & Comp Sci 2016; Available online: 12.05.2016.
% http://online.journals.tubitak.gov.tr/openAcceptedDocument.htm?fileID=696635&no=172411
% DOI: 10.3906/elk-1603-33 (This number will become active after the manuscript has been selected for inclusion in an issue.)
%
% This work is supported by the Scientific Research Project Fund of Cumhuriyet University under the project number TEKNO-002 
% Copyright (c) 2015 Cumhuriyet University
% Bu alma, Cumhuriyet niversitesi Bilimsel Aratrma Projeleri (CBAP) tarafndan TEKNO-002 proje numaras ile desteklenmitir.
% Telif Hakk (c) 2015 Cumhuriyet niversitesi
%
% Updates since the first version dated February 8, 2016:
% Nov 26, 2017: Lines 53 in this version is enclosed inside the if-then block in lines 52-56 in this version. This way, when fit_truncated.m
% fails to converge, KS P value is set as NaN and the search for valid solution candidates continues.
% Version date: Nov 26, 2017. Update was made possible thanks to data provided by Assoc. Prof. Dr. LKAY ULUSOY.
% Jan 25, 2018: Lines 47-48 in this version are altered such that truncation_thresholds do not exceed min(data) or max(data). min_data and
% max_data are introduced as global variables to avoid computing them at every iteration. They are computed once in ke_compute.m
% Version date: Jan 25, 2018.

global SCS data med solution_type low_med high_med KS_P 
global min_data max_data %m 25.01.2018

candidate=median(SCS); %Find the median of the current set of candidate solutions
[~, ind]=min(abs(SCS-candidate)); %Find the sample closest to the median
candidate=SCS(ind); %Use it as the solution candidate

switch solution_type
    case 'low_med'
        lower_limit=candidate; %Try this as the lower truncation threshold candidate
        upper_limit=med;
    case 'high_med'
        lower_limit=med; 
        upper_limit=candidate; %Try this as the upper truncation threshold candidate
    case 'scaling'
        lower_limit=max(med*(1-candidate)+low_med.solution*candidate, min_data); %Try this as a candidate for the scaling factor %m 25.01.2018
        upper_limit=min(med*(1-candidate)+high_med.solution*candidate, max_data); %Try this as a candidate for the scaling factor %m 25.01.2018
end
truncated_data=data(data>=lower_limit & data<=upper_limit); %try this interval
[mu, sigma, confidence_interval]=fit_truncated(truncated_data);
if ~isnan(mu) %m 25.11.2017
    P=compute_KS_P(truncated_data, mu, sigma);
else%m 25.11.2017
    P=NaN;%m 25.11.2017
end%m 25.11.2017

output.solution=candidate;
output.accepted=P>=KS_P;
output.value=P;
switch solution_type
    case 'low_med'
        output.success=med-candidate;
        output.more_info.low_med=[lower_limit, med];
    case 'high_med'
        output.success=candidate-med;
        output.more_info.high_med=[med, upper_limit];
    case 'scaling'
        output.success=upper_limit-lower_limit;
        output.more_info.truncation_thresholds=[lower_limit, upper_limit];
end
output.more_info.mu=mu;
output.more_info.sigma=sigma;
output.more_info.confidence_interval=confidence_interval;

%Update the set of candidate solutions based on the result
switch solution_type
    case 'low_med'
        if output.accepted, 
            SCS=SCS(SCS<candidate);
        else
            SCS=SCS(SCS>candidate);
        end
    case {'high_med', 'scaling'}
        if output.accepted, 
            SCS=SCS(SCS>candidate);
        else
            SCS=SCS(SCS<candidate);
        end
end