function truncation_thresholds=ke_compute_uni(varargin)
% ke_compute_uni
%
% 'ke_compute_uni' computes the unilateral truncation thresholds for continuous-valued time series data. It defines the set 
% of candidate solutions and the functions that will be used to find the solution. It forwards those to the function 
% 'bisection', which uses the bisection method to find the solution. It defines the global variables needed by other 
% functions in its body or in the function 'configure' defined below. data and KS P values can be optionally passed as input
% arguments. Otherwise it asks for data files and uses the P value defined in config_file as before.
% Adapted from the ke_compute.m version dated Jan 25, 2018  
%
% Author: Murat Okatan, PhD.
% Related Citation: Okatan M, Kocaturk M Truncation thresholds: a pair of spike detection thresholds computed using truncated 
% probability distributions. Turk J Elec Eng & Comp Sci. 2017, 25: 1436-1447. Available online: 12.05.2016. 
% DOI: 10.3906/elk-1603-33. http://online.journals.tubitak.gov.tr/openAcceptedDocument.htm?fileID=696635&no=172411
%
% This work is supported by the Turkish Scientific and Technical Research Council-TUBITAK under the project number 217E122 
% Copyright (c) 2018 TUBITAK
% Bu calisma, TUBITAK tarafindan 217E122 proje numarasi ile desteklenmistir.
% Telif Hakki (c) 2018 TUBITAK
%
% Updates
% 01.06.2019: added varargin{2,3}, which serve as the lower and upper KS P
% values, respectively. If varargin{2} exists without varargin{3}, then varargin{2} is used for both lower and upper truncation threshold. 

clear global message_handle SCS data med solution_type low_med high_med KS_P min_data max_data
clear min_data max_data 
clear KS_P KS_P_low KS_P_high %01.06.2019

global SCS %The Set of Candidate Solutions, which may be large, is defined as a global variable to avoid wasting time in passing it as an argument between functions.
global data med solution_type low_med high_med %Variables that 'evaluator' needs to know
global min_data max_data %Variables that 'evaluator' needs to know 
global KS_P KS_P_low KS_P_high %01.06.2019



truncation_thresholds=[];

%Read the config_file.txt and configure
configure;
KS_P_low=KS_P; %01.06.2019
KS_P_high=KS_P; %01.06.2019

no_input_file=0;
if nargin==0
    %Get the data file. If it is a *.mat file, it must contain a variable called 'data'. If it is a *.txt file,
    %the data are assumed to be in a single column format
    announce('Asking for data file(s)');
    [file_names, file_location] = uigetfile({'*.txt';'*.mat'}, 'Please select one or more data files', 'Multiselect', 'on');
    if isequal(file_names,0)
        exit_tasks('Data file not selected. Terminating.', '', truncation_thresholds);
        return
    end
    
    %Process all files
    if ischar(file_names)
        %One file was selected
        num_of_files=1;
        file_names={file_names};
    else
        num_of_files=length(file_names);
    end
end
if nargin>=1 %01.06.2019
    data=varargin{1};
    num_of_files=1;
    no_input_file=1;
    fname='';
end
if nargin>=2 %01.06.2019
    KS_P=varargin{2};
    KS_P_low=KS_P;
    KS_P_high=KS_P;
end
if nargin>=3 %01.06.2019
    KS_P_low=varargin{2};
    KS_P_high=varargin{3};
end

for fn=1:num_of_files
    
    if ~no_input_file
        %Current file
        file_name=file_names{fn};

        %Find the file extension
        [~,~,ext]=fileparts(file_name);
        fname=fullfile(file_location, file_name);
        announce(sprintf('Processing file %s.', fname));
        switch ext
            case '.mat'
                load(fname, 'data');
            case '.txt'
                fh=fopen(fname, 'rt');
                if fh==-1
                    exit_tasks('Could not open file.', '', truncation_thresholds, fn==num_of_files);
                    if fn==num_of_files
                        return
                    else
                        continue
                    end
                end
                data=textscan(fh, '%f');
                fclose(fh);
                data=data{1};
            otherwise
                exit_tasks('Data file extension must be *.txt or *.mat. Please check the user guide. Terminating.', '', truncation_thresholds, fn==num_of_files)
                if fn==num_of_files
                    return
                else
                    continue
                end
        end
    end

    data=reshape(data, length(data), 1);
    
    min_data=min(data);
    max_data=max(data);
    
    if isempty(data)
        %For some reason 'data' was not read successfully
        exit_tasks('Unable to read data. Terminating.', '', truncation_thresholds, fn==num_of_files);
        if fn==num_of_files
            return
        else
            continue
        end
    end

    %Let's first find the value of low|med
    med=0; %We are computing the truncation thresholds unilaterally. So we set the median to zero and calculate the lower 
           %truncation threshold first. Then we calculate the upper truncation threshold below.
    if isnan(KS_P_low) %01.06.2019
        low_med.solution=[]; %01.06.2019
        low_med.duration=0; %01.06.2019
    else %01.06.2019
        tic;
        KS_P=KS_P_low; %01.06.2019
        solution_type='low_med';
        announce(sprintf('Truncation Thresholds: Computing low|med for file %d/%d', fn, num_of_files), 'Title');
    %    med=median(data);
        SCS=unique(data(data<med));
        low_med=bisection(@evaluator);
        if isempty(low_med)
            %Solution was not found
            low_med.solution=med;
            low_med.duration=toc;
        else
            low_med.duration=toc;
        end
    end %01.06.2019

    %Then, let's find the value of high|med
    if isnan(KS_P_high)  %01.06.2019
        high_med.solution=[]; %01.06.2019
        high_med.duration=0; %01.06.2019
    else %01.06.2019
        tic;
        KS_P=KS_P_high; %01.06.2019
        solution_type='high_med';
        announce(sprintf('Truncation Thresholds: Computing high|med for file %d/%d', fn, num_of_files), 'Title');
        SCS=unique(data(data>med));
        high_med=bisection(@evaluator);
        if isempty(high_med)
            %Solution was not found
            high_med.solution=med;
            high_med.duration=toc;
        else
            high_med.duration=toc;
        end
    end %01.06.2019

    %At this point we have our unilateral truncation thresholds.        
    truncation_thresholds.low_med=low_med;
    truncation_thresholds.high_med=high_med;

    %Measure the duration
    duration=low_med.duration+high_med.duration;
    if ~isdeployed
        fprintf('Duration: %f min\n', duration/60);
    end
    
    %Exit
    exit_tasks(sprintf('Duration: %f min', duration/60), fname, truncation_thresholds, fn==num_of_files);
end

%%%%%%%%%%%%%%%%%%%
%    exit_tasks   %
%%%%%%%%%%%%%%%%%%%
function exit_tasks(textdata, fname, truncation_thresholds, varargin)
% exit_tasks(textdata, fname, truncation_thresholds, delete_box)
%
%Performs the cleanup operations when terminating
%
%textdata: The announcement made when terminating
%fname: Data file name. The output file name is derived from it.
%truncation_thresholds: The truncation thresholds that are computed
%varargin{1}: 1 if the message box is to be deleted, 0 otherwise

global message_handle %Handles related to the message box if provide_info=1 in config_file.txt
global med %to save the value of the median

delete_box=1;
if nargin>3
    delete_box=varargin{1};
end
    
%If file name is empty, data file was probably not read. Exit without saving.
if ~isempty(fname)
    %Write the median to the output
    truncation_thresholds.more_info.median=med;

    %Name the output file
    [pathdata, namedata, extdata]=fileparts(fname);
    output_fname=fullfile(pathdata, strcat(namedata, '_KE', extdata));

    switch extdata
        case '.mat'
            save(output_fname, 'truncation_thresholds');
        case '.txt'
            fh=fopen(output_fname, 'wt');
            if fh==-1
                announce('Could not open output file.');
            else
                if isempty(truncation_thresholds)
                    fprintf(fh, 'Could not find the truncation thresholds.\n');
                else
                    fprintf(fh, 'Truncation Thresholds:\n%f\t%f\n\n', truncation_thresholds.more_info.truncation_thresholds(1), truncation_thresholds.more_info.truncation_thresholds(2));
                    fprintf(fh, 'truncation_thresholds\n');
                    write_struct(truncation_thresholds, fh, 1);
                end
            end
            fclose(fh);
    end
end

announce(textdata);
pause(1);
if ishandle(message_handle.box) && delete_box
    delete(message_handle.box);
end


%%%%%%%%%%%%%%%%%%
%    configure   %
%%%%%%%%%%%%%%%%%%
function configure
% configure
%
%Configures the operation by running the commands in config_file.txt

global message_handle %Handles related to the message box if provide_info=1 in config_file.txt
global KS_P %The desired P-value for the Kolmogorov-Smirnov test

%The values in config_file.txt overwrite the values set below
announcement_FS=14; %Font Size for the announcements.
provide_info=1; %To follow the course of the operation set provide_info=1. No announcements are made otherwise.
KS_P=0.05; %The desired P-value for the Kolmogorov-Smirnov test

%Search for config_file.txt in the directory where the application was launched
file=fopen(fullfile(pwd, 'config_file.txt'), 'rt');

if file==-1
    warning('Failed to open config_file.txt. Using default configuration.');
    fprintf('Failed to open config_file.txt. Using default configuration.\n');
else
    commanddata=fgetl(file);
    while ischar(commanddata)
        [variable, remainder]=strtok(commanddata);
        if numel(variable)>0 && ~isequal(variable(1), '%')
            switch variable
                case {'announcement_FS','provide_info','KS_P'}
                    [value, ~]=strtok(remainder);
                    %verify that the value is scalar
                    if numel(str2num(value))~=1
                        fprintf('config_file.txt variable %s must be scalar\n', variable);
                    else
                        eval(strcat(variable, '=', value, ';'));
                    end
                otherwise
                    fprintf('config_file.txt variable %s not recognized\n', variable);
            end
        end
        commanddata=fgetl(file);
    end
end

if provide_info
    %Position the message box
    rt=groot;
    pos=rt.ScreenSize;
    G=round(pos(3)/21);
    Y=round(pos(4)/11);
    pos=[G 5*Y 13*G Y];
    
    %Select logo
    cdata=[];
    try
        [cdata,map] = imread('KE_logo.png');
    catch
        fprintf('Could not find the logo image.\n');
    end
    
    %Make the first announcement, format message box, get the handles
    if isempty(cdata)
        message_handle.box=msgbox('configure: Configured', 'Truncation Thresholds');
    else
        message_handle.box=msgbox('configure: Configured', 'Truncation Thresholds', 'custom', cdata, map);
    end
    set(message_handle.box, 'Resize', 'on', 'Position', pos);
    at=get(message_handle.box, 'Children');
    message_handle.textdata=get(at(end), 'Children');
    set(message_handle.textdata, 'FontSize', announcement_FS);
else
    message_handle.box=NaN;
    message_handle.textdata=NaN;
end


%%%%%%%%%%%%%%%%%%%%%
%    write_struct   %
%%%%%%%%%%%%%%%%%%%%%
function write_struct(st, fh, tabnum)

L_st=length(st);

for j=1:L_st
    if L_st>1
        fprintf(fh, '%d', j);
    end
    fn=fieldnames(st(j));
    L=length(fn);
    for i=1:L
        dummy=getfield(st(j), fn{i});
        if isstruct(dummy)
            for t=1:tabnum
                fprintf(fh, '\t');
            end
            fprintf(fh, '%s\n', fn{i});
            write_struct(dummy, fh, tabnum+1);
        else
            for t=1:tabnum
                fprintf(fh, '\t');
            end
            fprintf(fh, '%s: ', fn{i});
            fprintf(fh, '%f ', dummy);        
            fprintf(fh, '\n');
        end
    end
    fprintf(fh, '\n');
end